@@ -21,6 +21,9 @@
 #include "mask.xbm"
 #include "sonypi.h"
 #include "acpi.h"
+#ifdef HAL
+#include "simplehal.h"
+#endif
 
 Pixmap images[NUM_IMAGES];
 Window root, iconwin, win;
@@ -35,6 +38,9 @@ char *crit_audio;
 int crit_audio_size;
 
 int battnum = 1;
+#ifdef HAL
+int use_simplehal = 0;
+#endif
 int use_sonypi = 0;
 int use_acpi = 0;
 int delay = 0;
@@ -242,7 +248,7 @@ char *parse_commandline(int argc, char *argv[]) {
 		  case 'c':
 			critical_pct = atoi(optarg);
 			break;
-		  case 'r':
+		  case 'e':
 			always_estimate_remaining = 1;
 			break;
 		  case 'a':
@@ -509,6 +515,12 @@ void alarmhandler(int sig) {
 		if (acpi_read(battnum, &cur_info) != 0)
 			error("Cannot read ACPI information.");
 	}
+#ifdef HAL
+	else if (use_simplehal) {
+		if (simplehal_read(battnum, &cur_info) != 0)
+			error("Cannot read HAL information.");
+	}
+#endif
 	else if (! use_sonypi) {
 		if (apm_read(&cur_info) != 0)
 			error("Cannot read APM information.");
@@ -551,6 +563,16 @@ void alarmhandler(int sig) {
 	alarm(delay);
 }
 
+void check_battery_num(int real, int requested) {
+	if (requested > real || requested < 1) {
+		error("There %s only %i batter%s, and you asked for number %i.",
+			real == 1 ? "is" : "are",
+			real,
+			real == 1 ? "y" : "ies",
+			requested);
+	}
+}
+
 int main(int argc, char *argv[]) {
 	make_window(parse_commandline(argc, argv), argc ,argv);
 
@@ -559,18 +581,20 @@ int main(int argc, char *argv[]) {
 		if (! delay)
 			delay = 1;
 	}
+#ifdef HAL
+	/* Check for hal support. */
+	else if (simplehal_supported()) {
+		use_simplehal = 1;
+		if (! delay)
+			delay = 2;
+	}
+#endif
 	/* Check for ACPI support. */
 	else if (acpi_supported() && acpi_batt_count > 0) {
-		if (battnum > acpi_batt_count || battnum < 1) {
-			error("There %s only %i batter%s, and you asked for number %i.",
-					acpi_batt_count == 1 ? "is" : "are",
-					acpi_batt_count,
-					acpi_batt_count == 1 ? "y" : "ies",
-					battnum);
-		}
+		check_battery_num(acpi_batt_count, battnum);
 		use_acpi = 1;
 		if (! delay)
-			delay = 3; /* sigh */
+			delay = 3; /* slow interface! */
 	}
 	else if (sonypi_supported()) {
 		use_sonypi = 1;
@@ -580,8 +604,7 @@ int main(int argc, char *argv[]) {
 			delay = 1;
 	}
 	else {
-	
-		error("No APM, ACPI, or SPIC support in kernel.");
+		error("No APM, ACPI, HAL or SPIC support detected.");
 	}
 		
 	load_images();
